<?php
/**
 * Hero columns Widget
 *
 * @package bolts-pt
 */

if ( ! class_exists( 'PW_Hero_Columns' ) ) {
	class PW_Hero_Columns extends WP_Widget {

		// Basic widget settings.
		function widget_id_base() { return 'hero_columns'; }
		function widget_name() { return esc_html__( 'Hero Columns', 'bolts-pt' ); }
		function widget_description() { return esc_html__( 'Multiple hero columns with text and background images for Page Builder.', 'bolts-pt' ); }
		function widget_class() { return 'widget-hero-columns'; }

		public function __construct() {
			parent::__construct(
				'pw_' . $this->widget_id_base(),
				sprintf( 'ProteusThemes: %s', $this->widget_name() ),
				array(
					'description' => $this->widget_description(),
					'classname'   => $this->widget_class(),
				)
			);
		}

		/**
		 * Front-end display of widget.
		 *
		 * @see WP_Widget::widget()
		 *
		 * @param array $args
		 * @param array $instance
		 */
		public function widget( $args, $instance ) {

			// Prepare data.
			$bg_opacity = empty( $instance['bg_opacity'] ) ? 0 : ( absint( $instance['bg_opacity'] ) / 100 );
			$items      = isset( $instance['items'] ) ? $instance['items'] : array();
			$widget_id  = 'hero-columns-' . $args['widget_id'];

			echo $args['before_widget'];
			?>
				<div class="hero-columns" id="<?php echo esc_attr( $widget_id ); ?>">
					<?php foreach ( $items as $item ) : ?>
						<div class="hero-columns__item" style="background-image: url('<?php echo esc_url( $item['bg_image'] ); ?>')">
							<div class="hero-columns__content">
									<h3 class="hero-columns__title"><?php echo wp_kses_post( $item['title'] ); ?></h3>
									<div class="hero-columns__text">
										<?php echo wp_kses_post( $item['content'] ); ?>
									</div>
							</div>
							<a class="hero-columns__cta" href="<?php echo esc_url( $item['link_url'] ); ?>" target="<?php echo empty( $instance['new_tab'] ) ? '_self' : '_blank'; ?>">
								<?php echo esc_html( $item['link_text'] ); ?>
							</a>
							<div class="hero-columns__overlay" style="background-color: rgba(0, 0, 0, <?php echo esc_attr( $bg_opacity ); ?>)"></div>
						</div>
					<?php endforeach; ?>
				</div>
			<?php
			echo $args['after_widget'];
		}

		/**
		 * Sanitize widget form values as they are saved.
		 *
		 * @param array $new_instance The new options.
		 * @param array $old_instance The previous options.
		 */
		public function update( $new_instance, $old_instance ) {
			$instance = array();

			$instance['bg_opacity'] = absint( BoltsHelpers::bound( $new_instance['bg_opacity'], 0, 100 ) );
			$instance['new_tab']    = ! empty( $new_instance['new_tab'] ) ? sanitize_key( $new_instance['new_tab'] ) : '';

			if ( ! empty( $new_instance['items'] )  ) {
				foreach ( $new_instance['items'] as $key => $item ) {
					$instance['items'][ $key ]['id']        = sanitize_key( $item['id'] );
					$instance['items'][ $key ]['title']     = wp_kses_post( $item['title'] );
					$instance['items'][ $key ]['content']   = wp_kses_post( $item['content'] );
					$instance['items'][ $key ]['bg_image']  = esc_url_raw( $item['bg_image'] );
					$instance['items'][ $key ]['link_text'] = sanitize_text_field( $item['link_text'] );
					$instance['items'][ $key ]['link_url']  = esc_url_raw( $item['link_url'] );
				}
			}

			return $instance;
		}

		/**
		 * Back-end widget form.
		 *
		 * @param array $instance The widget options.
		 */
		public function form( $instance ) {
			$bg_opacity = empty( $instance['bg_opacity'] ) ? 0 : $instance['bg_opacity'];
			$new_tab    = empty( $instance['new_tab'] ) ? '' : $instance['new_tab'];
			$items      = isset( $instance['items'] ) ? $instance['items'] : array();

			// Page Builder fix when using repeating fields.
			if ( 'temp' === $this->id ) {
				$this->current_widget_id = $this->number;
			}
			else {
				$this->current_widget_id = $this->id;
			}
			?>

			<p>
				<label for="<?php echo esc_attr( $this->get_field_id( 'bg_opacity' ) ); ?>"><?php esc_html_e( 'Opacity:', 'bolts-pt' ); ?></label>
				<input id="<?php echo esc_attr( $this->get_field_id( 'bg_opacity' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'bg_opacity' ) ); ?>" type="number" min="0" max="100" step="1" value="<?php echo esc_attr( $bg_opacity ); ?>" /><br>
				<small><?php esc_html_e( 'Input a percentage for the background opacity. From 0 to 100.', 'bolts-pt' )?></small>
			</p>

			<p>
				<input class="checkbox" type="checkbox" <?php checked( $new_tab, 'on' ); ?> id="<?php echo esc_attr( $this->get_field_id( 'new_tab' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'new_tab' ) ); ?>" value="on" />
				<label for="<?php echo esc_attr( $this->get_field_id( 'new_tab' ) ); ?>"><?php esc_html_e( 'Open links in new tab', 'bolts-pt' ); ?></label>
			</p>

			<hr>

			<h3><?php esc_html_e( 'Hero Columns:', 'bolts-pt' ); ?></h3>

			<script type="text/template" id="js-pt-hero-column-<?php echo esc_attr( $this->current_widget_id ); ?>">
				<p>
					<label for="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-title"><?php esc_html_e( 'Title:', 'bolts-pt' ); ?></label>
					<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-title" name="<?php echo esc_attr( $this->get_field_name( 'items' ) ); ?>[{{id}}][title]" type="text" value="{{title}}" />
				</p>
				<p>
					<label for="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-content"><?php esc_html_e( 'Content:', 'bolts-pt' ); ?></label>
					<textarea rows="4" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-content" name="<?php echo esc_attr( $this->get_field_name( 'items' ) ); ?>[{{id}}][content]">{{content}}</textarea>
				</p>
				<p>
					<label for="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-bg_image"><?php esc_html_e( 'Background image URL:', 'bolts-pt' ); ?></label>
					<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-bg_image" name="<?php echo esc_attr( $this->get_field_name( 'items' ) ); ?>[{{id}}][bg_image]" type="text" value="{{bg_image}}" />
					<input type="button" onclick="ProteusWidgetsUploader.imageUploader.openFileFrame('<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-bg_image');" class="upload-background-image button button-secondary pull-right" value="<?php esc_html_e( 'Upload image', 'bolts-pt' ); ?>" /> <!-- Media uploader button -->
				</p>
				<p>
					<label for="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-link_text"><?php esc_html_e( 'Link text:', 'bolts-pt' ); ?></label>
					<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-link_text" name="<?php echo esc_attr( $this->get_field_name( 'items' ) ); ?>[{{id}}][link_text]" type="text" value="{{link_text}}" />
				</p>
				<p>
					<label for="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-link_url"><?php esc_html_e( 'Link URL:', 'bolts-pt' ); ?></label>
					<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'items' ) ); ?>-{{id}}-link_url" name="<?php echo esc_attr( $this->get_field_name( 'items' ) ); ?>[{{id}}][link_url]" type="text" value="{{link_url}}" />
				</p>
				<p>
					<input name="<?php echo esc_attr( $this->get_field_name( 'items' ) ); ?>[{{id}}][id]" type="hidden" value="{{id}}" />
					<a href="#" class="pt-remove-hero-column  js-pt-remove-hero-column"><span class="dashicons dashicons-dismiss"></span> <?php esc_html_e( 'Remove item', 'bolts-pt' ); ?></a>
				</p>
			</script>
			<div class="pt-widget-hero-columns" id="hero-columns-<?php echo esc_attr( $this->current_widget_id ); ?>">
				<div class="hero-columns"></div>
				<p>
					<a href="#" class="button  js-pt-add-hero-column"><?php esc_html_e( 'Add New Item', 'bolts-pt' ); ?></a>
				</p>
			</div>

			<script type="text/javascript">
				(function() {
					// repopulate the form
					var heroColumnsJSON = <?php echo wp_json_encode( $items ) ?>;

					// get the right widget id and remove the added < > characters at the start and at the end.
					var widgetId = '<<?php echo esc_js( $this->current_widget_id ); ?>>'.slice( 1, -1 );

					if ( _.isFunction( Bolts.Utils.repopulateHeroColumns ) ) {
						Bolts.Utils.repopulateHeroColumns( heroColumnsJSON, widgetId );
					}
				})();
			</script>

			<?php
		}
	}
	register_widget( 'PW_Hero_Columns' );
}
